@@ -171,76 +171,8 @@ class VBGMR(BayesianGaussianMixture):
         weights : numpy.array
             m x l matrix of weights,
         """
-
-        dataset = np.array(dataset)
-        if dataset.ndim == 0:
-            dataset = np.reshape(dataset, (1, 1))
-        elif dataset.ndim == 1:
-            dataset = np.reshape(dataset, (1, dataset.shape[0]))
-
-        input_means = self.means_[:, numbers_of_input_variables]
-        output_means = self.means_[:, numbers_of_output_variables]
-
-        if self.covariance_type == 'full':
-            all_covariances = self.covariances_
-        elif self.covariance_type == 'diag':
-            all_covariances = np.empty(
-                [self.n_components, self.covariances_.shape[1], self.covariances_.shape[1]])
-            for component_number in range(self.n_components):
-                all_covariances[component_number, :, :] = np.diag(self.covariances_[component_number, :])
-        elif self.covariance_type == 'tied':
-            all_covariances = np.tile(self.covariances_, (self.n_components, 1, 1))
-        elif self.covariance_type == 'spherical':
-            all_covariances = np.empty([self.n_components, self.means_.shape[1], self.means_.shape[1]])
-            for component_number in range(self.n_components):
-                all_covariances[component_number, :, :] = np.diag(
-                    self.covariances_[component_number] * np.ones(self.means_.shape[1]))
-
-#        print(all_covariances.shape[2], len(numbers_of_input_variables), len(numbers_of_output_variables))
-        if all_covariances.shape[2] == len(numbers_of_input_variables) + len(numbers_of_output_variables):
-            input_output_covariances = all_covariances[:, numbers_of_input_variables, :]
-            input_covariances = input_output_covariances[:, :, numbers_of_input_variables]
-            input_output_covariances = input_output_covariances[:, :, numbers_of_output_variables]
-
-            # estimated means and weights for all components
-            estimated_mean_for_all_components = np.empty(
-                [self.n_components, dataset.shape[0], len(numbers_of_output_variables)])
-            weights = np.empty([self.n_components, dataset.shape[0]])
-            for component_number in range(self.n_components):
-                estimated_mean_for_all_components[component_number, :, :] = output_means[component_number, :] + (
-                        dataset - input_means[component_number, :]).dot(
-                    np.linalg.inv(input_covariances[component_number, :, :])).dot(
-                    input_output_covariances[component_number, :, :])
-
-                weights[component_number, :] = self.weights_[component_number] * \
-                                               multivariate_normal.pdf(dataset,
-                                                                       input_means[component_number, :],
-                                                                       input_covariances[component_number, :, :], allow_singular=True)
-            if len(np.where(weights.sum(axis=0)==0)[0]) > 0:
-                weights = np.ones(weights.shape)
-            if np.isnan(weights.sum(axis=0)).any():
-                weights = np.ones(weights.shape)
-            if np.isinf(weights.sum(axis=0)).any():
-                weights = np.ones(weights.shape)
-            weights = weights / weights.sum(axis=0)
-
-            # calculate mode of estimated means and weighted estimated means
-            mode_of_estimated_mean = np.empty([dataset.shape[0], len(numbers_of_output_variables)])
-            weighted_estimated_mean = np.empty([dataset.shape[0], len(numbers_of_output_variables)])
-            index_for_mode = np.argmax(weights, axis=0)
-            for sample_number in range(dataset.shape[0]):
-                mode_of_estimated_mean[sample_number, :] = estimated_mean_for_all_components[
-                                                           index_for_mode[sample_number],
-                                                           sample_number, :]
-                weighted_estimated_mean[sample_number, :] = weights[:, sample_number].dot(
-                    estimated_mean_for_all_components[:, sample_number, :])
-        else:
-            mode_of_estimated_mean = np.ones([dataset.shape[0], len(numbers_of_output_variables)]) * -99999
-            weighted_estimated_mean = np.ones([dataset.shape[0], len(numbers_of_output_variables)]) * -99999
-            weights = np.zeros([self.n_components, dataset.shape[0]])
-            estimated_mean_for_all_components = np.zeros(
-                [self.n_components, dataset.shape[0], len(numbers_of_output_variables)])
-
+        
+        mode_of_estimated_mean, weighted_estimated_mean, _, _ = self.predict(dataset, numbers_of_input_variables, numbers_of_output_variables)
         if self.rep == 'mean':
             values = mode_of_estimated_mean.copy()
         elif self.rep == 'mode':
@@ -289,6 +221,15 @@ class VBGMR(BayesianGaussianMixture):
         autoscaled_dataset = (dataset - dataset.mean(axis=0)) / dataset.std(axis=0, ddof=1)
         reps = ['mean', 'mode']
         
+        min_number = math.floor(dataset.shape[0] / fold_number)
+        mod_number = dataset.shape[0] - min_number * fold_number
+        index = np.matlib.repmat(np.arange(1, fold_number + 1, 1), 1, min_number).ravel()
+        if mod_number != 0:
+            index = np.r_[index, np.arange(1, mod_number + 1, 1)]
+#        np.random.seed(999)
+        fold_index_in_cv = np.random.permutation(index)
+        np.random.seed()
+                                
         r2cvs = []
         hyperparameters = []
         for covariance_type in covariance_types:
@@ -303,15 +244,7 @@ class VBGMR(BayesianGaussianMixture):
                                 self.rep = rep
                                 hyperparameters.append([covariance_type, number_of_components, weight_concentration_prior_type, weight_concentration_prior, rep])
                                 estimated_y_in_cv = np.zeros([dataset.shape[0], len(numbers_of_output_variables)])
-                
-                                min_number = math.floor(dataset.shape[0] / fold_number)
-                                mod_number = dataset.shape[0] - min_number * fold_number
-                                index = np.matlib.repmat(np.arange(1, fold_number + 1, 1), 1, min_number).ravel()
-                                if mod_number != 0:
-                                    index = np.r_[index, np.arange(1, mod_number + 1, 1)]
-                #                np.random.seed(999)
-                                fold_index_in_cv = np.random.permutation(index)
-                                np.random.seed()
+                                
                                 for fold_number_in_cv in np.arange(1, fold_number + 1, 1):
                                     dataset_train_in_cv = autoscaled_dataset[fold_index_in_cv != fold_number_in_cv, :]
                                     dataset_test_in_cv = autoscaled_dataset[fold_index_in_cv == fold_number_in_cv, :]
@@ -338,4 +271,5 @@ class VBGMR(BayesianGaussianMixture):
         self.weight_concentration_prior_type = max_r2cv_hyperparameter[2]
         self.weight_concentration_prior = max_r2cv_hyperparameter[3]
         self.rep = max_r2cv_hyperparameter[4]
+        self.r2cv = r2cvs[max_r2cv_number]
         
