@@ -6,3 +6,5 @@
 
 from .fast_opt_svr_hyperparams import fast_opt_svr_hyperparams
 from .fast_opt_svr_hyperparams import maximize_variance_of_gram_matrix
+from .iot import iot_obj_func
+from .iot import iot
